{% extends (base or "manhattan/manage/emails/base.html") %}

{% block header_button %}{% endblock %}

{% block content %}
    <h1>A recovery code used to 2-factor authorize your account</h1>
    <p class="large-text">
        Hi {{ user.first_name }},

        One of your recovery codes was just used to 2-factor authorize your
        account.
    </p>

    <h3 class="h2">What if you didn't use the recovery code?</h3>
    <p>
        If you haven't used a recovery code to 2-factor authorize your account
        you need to notify your web development team as soon as possible as
        your account may have been hijacked.
    </p>
{% endblock %}
