{%- extends "manhattan/manage/generics/update.html" -%}
{%- import "manhattan/manage/components/form.html" as _form with context -%}

{%- block fieldsets -%}

    {% call _form.fieldset('About') -%}
        {% call _form.field() -%}
            {{ _form.control(form.first_name) }}
            {{ _form.control(form.last_name) }}
        {%- endcall %}
    {%- endcall %}

    {% call _form.fieldset('Sign-in details') -%}

        {% include "manhattan/users/includes/password_rules.html" %}

        {{ _form.field(form.email) }}
        {% call _form.field() -%}
            {{ _form.control(
                form.new_password,
                input_attrs={'autocomplete': 'new-password'}
            ) }}
            {{ _form.control(
                form.confirm_password,
                input_attrs={'autocomplete': 'new-password'}
             ) }}
        {%- endcall %}

    {%- endcall %}

{%- endblock %}
