
{% call _form.note('mh-formatted') %}
    {% set rules = config[manage_config.frame_cls.get_settings_prefix() + '_PASSWORD_RULES'] %}
    <b>Passwords must use</b>;
    {% if rules.min_lower %}
        {{ rules.min_lower }}
        lowercase character{% if rules.min_lower != 1 %}s{% endif %}
        [a-z],
    {% endif %}
    {% if rules.min_upper %}
        {{ rules.min_upper }}
        uppercase character{% if rules.min_upper != 1 %}s{% endif %}
        [A-Z],
    {% endif %}
    {% if rules.min_digits %}
        {{ rules.min_digits }}
        number{% if rules.min_digits != 1 %}s{% endif %}
        [0-9],
    {% endif %}
    {% if rules.min_specials %}
        {{ rules.min_specials }}
        non-alphanumeric
        character{% if rules.min_specials != 1 %}s{% endif %},
    {% endif %}
    and be at least {{ rules.min_length }} characters long.
{% endcall %}
