{% extends (base or "manhattan/manage/emails/base.html") %}

{% block header_button %}{% endblock %}

{% block content %}
    <h1>Your password reset request</h1>
    <p class="large-text">
        Hi {{ user.first_name }}, a password reset request has been made for
        your account. If you made this request, please click the link below to
        set a new password for your account.
    </p>

    {{ _email.buttons('Set new password', url=reset_link) }}
    {{ _email.divider() }}
    <h3 class="h2">Not sent by you?</h3>
    <p>
        If you did not make the password reset request then you don't need to do
        anything and you can simply ignore this email.
    </p>
{% endblock %}
