{%- extends "manhattan/manage/generics/add.html" -%}
{%- import "manhattan/manage/components/form.html" as _form with context -%}

{%- block fieldsets -%}

    {% call _form.fieldset('About') -%}
        {% call _form.field() -%}
            {{ _form.control(form.first_name) }}
            {{ _form.control(form.last_name) }}
        {%- endcall %}

    {%- endcall %}

    {% call _form.fieldset('Sign-in details') -%}
        {% call _form.aside() %}
            You can specify the email address that this user will use to
            login with below. Once added they will receive an email inviting
            them to the application.
        {% endcall %}
        {{ _form.field(form.email) }}
    {%- endcall %}

{%- endblock %}

