{%- extends "manhattan/manage/generics/add.html" -%}
{%- import "manhattan/manage/components/form.html" as _form with context -%}

{%- block fieldsets -%}

    {% call _form.fieldset() -%}

        {% call _form.note(class='mh-formatted') %}
            <p>
                To disable 2-factor authentication enter your current password
                below and click <em>Disable 2FA</em> button.
            </p>
        {% endcall %}

        {{ _form.field(
            form.current_password,
            class='mh-field--3-3'
        ) }}

    {%- endcall %}

{%- endblock %}

{%- block buttons -%}
    {{ _form.button('Disable 2FA') }}
{%- endblock -%}
