{% extends (base or "manhattan/manage/emails/base.html") %}

{% block header_button %}{% endblock %}

{% block content %}
    <h1>You are cordially invited</h1>
    <p class="large-text">
        Hi {{ user.first_name }},

        You've been invited to be an administrator for the {{ project_manager}}
        project. Once you accept your invite you'll be able to access and manage
        the project.
    </p>
    {{ _email.buttons('Accept invite', url=invite_link) }}
{% endblock %}
