{% extends "manhattan/manage/minimal.html" %}
{%- import "manhattan/manage/components/form.html" as _form with context -%}

{% block main -%}
    {% call _form.form(form) -%}
        {% call _form.fieldset() -%}
            {{ _form.field(form.email) }}
            {{ _form.buttons('Send') }}
        {%- endcall %}
    {%- endcall %}
{%- endblock %}

{% block footer -%}
    <a href="{{ url_for(manage_config.get_endpoint('sign_in')) }}">
        Remembered your password? Login
    </a>
{%- endblock %}

