{%- extends "manhattan/manage/generics/update.html" -%}
{%- import "manhattan/manage/components/form.html" as _form with context -%}

{%- block fieldsets -%}

    {% call _form.fieldset('About') -%}
        {% call _form.field() -%}
            {{ _form.control(form.first_name) }}
            {{ _form.control(form.last_name) }}
        {%- endcall %}
    {%- endcall %}

    {% call _form.fieldset('Contact details') -%}
        {{ _form.field(form.email) }}
    {%- endcall %}

{%- endblock %}
