{%- extends "manhattan/manage/generics/list.html" -%}
{%- import "manhattan/manage/components/status.html" as _status -%}

{% macro headings() -%}
    {{ _results.heading('Name', sort_with='full_name_lower') }}
{%- endmacro %}

{% macro columns(item) -%}
    {% call _results.column() %}
        {{ item.full_name }}
    {% endcall %}
{%- endmacro %}
