{% extends (base or "manhattan/manage/emails/base.html") %}

{% block header_button %}{% endblock %}

{% block content %}
    <h1>Your email address has been changed</h1>
    <p class="large-text">
        Hi {{ user.first_name }}, we wanted to let you know that your email
        address has been changed:
    </p>

    {%- call _email.content_list() -%}
        {{ _email.content_row('From', original_email) }}
        {{ _email.content_row('To', user.email) }}
    {%- endcall -%}

    <h3 class="h2">What if I didn't make this change?</h3>
    <p>
        If you didn't make this change then you need to notify your web
        development team as soon as possible as your account may have been
        hijacked.
    </p>
{% endblock %}
