{%- extends "manhattan/manage/generics/add.html" -%}
{%- import "manhattan/manage/components/form.html" as _form with context -%}

{%- block fieldsets -%}

    {% call _form.fieldset() -%}

        {% call _form.note(class='mh-formatted') %}
            <p>
                <b>Enter your six-digit code</b>
                <br>
                Open your registered 2-factor authentication app and enter the
                six-digit code displayed for <b>{{ mfa_issuer }}</b>.
            </p>
            <p>
                <em>If you don't have your authentication app available you can
                enter a recovery code instead.</em>
            </p>
        {% endcall %}

        {{ _form.field(form.totp, class='mh-field--3-3') }}

    {%- endcall %}

{%- endblock %}

{%- block buttons -%}
    {{ _form.button('Authorize') }}
{%- endblock -%}
