{% extends "manhattan/manage/minimal.html" %}
{%- import "manhattan/manage/components/form.html" as _form with context -%}

{% block main -%}
    {% call _form.form(form) %}
        {% call _form.fieldset() %}
            {{ _form.field(form.password) }}
            {{ _form.field(form.confirm_password) }}
            {% include "manhattan/users/includes/password_rules_minimal.html" %}
            {{ _form.buttons('Save') }}
        {% endcall %}
    {% endcall %}
{%- endblock %}
