{% extends (base or "manhattan/manage/emails/base.html") %}

{% block header_button %}{% endblock %}

{% block content %}

    <h1>A new device was used to sign-in</h1>
    <p class="large-text">
        Hi {{ user.first_name }}, we just wanted to let you know that a new
        device (IP address, Browser, Platform) has been used to log into your
        account.
    </p>

    {%- call _email.content_list() -%}
        {{ _email.content_row('Location', user_session.location) }}
        {{ _email.content_row('Platform', user_session.platform) }}
    {%- endcall -%}

    <h3 class="h2">What if I don't recognize this device?</h3>
    <p>
        If this isn't a device you recognize then you need to notify your web
        development team as soon as possible as your account may have been
        hijacked.
    </p>

{% endblock %}
