
{% call _form.aside('mh-formatted') %}
    {% set rules = config[manage_config.frame_cls.get_settings_prefix() + '_PASSWORD_RULES'] %}
    <b>Passwords must</b> use;
    <ul>
        {% if rules.min_lower %}
            <li>
                {{ rules.min_lower }}
                lowercase character{% if rules.min_lower != 1 %}s{% endif %}
                [a-z],
            </li>
        {% endif %}
        {% if rules.min_upper %}
            <li>
                {{ rules.min_upper }}
                uppercase character{% if rules.min_upper != 1 %}s{% endif %}
                [A-Z],
            </li>
        {% endif %}
        {% if rules.min_digits %}
            <li>
                {{ rules.min_digits }}
                number{% if rules.min_digits != 1 %}s{% endif %}
                [0-9],
            </li>
        {% endif %}
        {% if rules.min_specials %}
            <li>
                {{ rules.min_specials }}
                non-alphanumeric
                character{% if rules.min_specials != 1 %}s{% endif %},
            </li>
        {% endif %}
        <li>and be at least {{ rules.min_length }} characters long.</li>
    </ul>
{% endcall %}
